@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2007-2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2007-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -50,11 +50,11 @@ static gint compare_priority(gconstpointer a, gconstpointer b)
  *
  * Returns: %0 on success
  */
-int connman_notifier_register(struct connman_notifier *notifier)
+int connman_notifier_register(const struct connman_notifier *notifier)
 {
 	DBG("notifier %p name %s", notifier, notifier->name);
 
-	notifier_list = g_slist_insert_sorted(notifier_list, notifier,
+	notifier_list = g_slist_insert_sorted(notifier_list, (void*)notifier,
 							compare_priority);
 
 	return 0;
@@ -66,24 +66,22 @@ int connman_notifier_register(struct connman_notifier *notifier)
  *
  * Remove a previously registered notifier module
  */
-void connman_notifier_unregister(struct connman_notifier *notifier)
+void connman_notifier_unregister(const struct connman_notifier *notifier)
 {
 	DBG("notifier %p name %s", notifier, notifier->name);
 
 	notifier_list = g_slist_remove(notifier_list, notifier);
 }
 
-#define MAX_TECHNOLOGIES 10
-
-static int connected[MAX_TECHNOLOGIES];
-static int online[MAX_TECHNOLOGIES];
+static int connected[MAX_CONNMAN_SERVICE_TYPES];
+static int online[MAX_CONNMAN_SERVICE_TYPES];
 
 static bool notifier_is_online(void)
 {
 	unsigned int i;
 
 	__sync_synchronize();
-	for (i = 0; i < MAX_TECHNOLOGIES; i++) {
+	for (i = 0; i < MAX_CONNMAN_SERVICE_TYPES; i++) {
 		if (online[i] > 0)
 			return true;
 	}
@@ -96,7 +94,7 @@ bool __connman_notifier_is_connected(void)
 	unsigned int i;
 
 	__sync_synchronize();
-	for (i = 0; i < MAX_TECHNOLOGIES; i++) {
+	for (i = 0; i < MAX_CONNMAN_SERVICE_TYPES; i++) {
 		if (connected[i] > 0)
 			return true;
 	}
@@ -148,12 +146,13 @@ void __connman_notifier_connect(enum connman_service_type type)
 	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_GPS:
 	case CONNMAN_SERVICE_TYPE_VPN:
-	case CONNMAN_SERVICE_TYPE_GADGET:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_GADGET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
+	case CONNMAN_SERVICE_TYPE_P2P:
 		break;
 	}
 
@@ -194,12 +193,13 @@ void __connman_notifier_disconnect(enum connman_service_type type)
 	case CONNMAN_SERVICE_TYPE_SYSTEM:
 	case CONNMAN_SERVICE_TYPE_GPS:
 	case CONNMAN_SERVICE_TYPE_VPN:
-	case CONNMAN_SERVICE_TYPE_GADGET:
 		return;
 	case CONNMAN_SERVICE_TYPE_ETHERNET:
+	case CONNMAN_SERVICE_TYPE_GADGET:
 	case CONNMAN_SERVICE_TYPE_WIFI:
 	case CONNMAN_SERVICE_TYPE_BLUETOOTH:
 	case CONNMAN_SERVICE_TYPE_CELLULAR:
+	case CONNMAN_SERVICE_TYPE_P2P:
 		break;
 	}
 
@@ -215,7 +215,7 @@ void __connman_notifier_default_changed(struct connman_service *service)
 	GSList *list;
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->default_changed)
 			notifier->default_changed(service);
@@ -228,7 +228,7 @@ void __connman_notifier_service_add(struct connman_service *service,
 	GSList *list;
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->service_add)
 			notifier->service_add(service, name);
@@ -251,7 +251,7 @@ void __connman_notifier_service_remove(struct connman_service *service)
 	}
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->service_remove)
 			notifier->service_remove(service);
@@ -263,7 +263,7 @@ void __connman_notifier_proxy_changed(struct connman_service *service)
 	GSList *list;
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->proxy_changed)
 			notifier->proxy_changed(service);
@@ -289,7 +289,7 @@ void __connman_notifier_offlinemode(bool enabled)
 	state_changed();
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->offline_mode)
 			notifier->offline_mode(enabled);
@@ -303,7 +303,7 @@ static void notify_idle_state(bool idle)
 	DBG("idle %d", idle);
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->idle_state)
 			notifier->idle_state(idle);
@@ -318,7 +318,7 @@ void __connman_notifier_service_state_changed(struct connman_service *service,
 	bool found;
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->service_state_changed)
 			notifier->service_state_changed(service, state);
@@ -361,7 +361,7 @@ void __connman_notifier_ipconfig_changed(struct connman_service *service,
 	GSList *list;
 
 	for (list = notifier_list; list; list = list->next) {
-		struct connman_notifier *notifier = list->data;
+		const struct connman_notifier *notifier = list->data;
 
 		if (notifier->ipconfig_changed)
 			notifier->ipconfig_changed(service, ipconfig);
