@@ -28,8 +28,6 @@
 
 #include <gdbus.h>
 
-#include <connman/session.h>
-
 #include "connman.h"
 
 static DBusConnection *connection;
@@ -515,6 +513,9 @@ static void free_session(struct connman_session *session)
 	if (session->notify_watch > 0)
 		g_dbus_remove_watch(connection, session->notify_watch);
 
+	g_dbus_unregister_interface(connection, session->session_path,
+				    CONNMAN_SESSION_INTERFACE);
+
 	destroy_policy_config(session);
 	g_slist_free(session->info->config.allowed_bearers);
 	g_free(session->info->config.allowed_interface);
@@ -1407,8 +1408,9 @@ static int session_policy_config_cb(struct connman_session *session,
 					session_methods, NULL, NULL,
 					session, NULL)) {
 		connman_error("Failed to register %s", session->session_path);
+		g_hash_table_remove(session_hash, session->session_path);
 		err = -EINVAL;
-		goto err;
+		goto err_notify;
 	}
 
 	reply = g_dbus_create_reply(creation_data->pending,
@@ -1433,12 +1435,13 @@ static int session_policy_config_cb(struct connman_session *session,
 	return 0;
 
 err:
-	g_hash_table_remove(session_hash, session->session_path);
+	cleanup_session(session);
+
+err_notify:
 	reply = __connman_error_failed(creation_data->pending, -err);
 	g_dbus_send_message(connection, reply);
 	creation_data->pending = NULL;
 
-	cleanup_session(session);
 	cleanup_creation_data(creation_data);
 
 	return err;
@@ -1712,7 +1715,7 @@ static void update_session_state(struct connman_session *session)
 	enum connman_session_state state = CONNMAN_SESSION_STATE_DISCONNECTED;
 
 	if (session->service) {
-		service_state = __connman_service_get_state(session->service);
+		service_state = connman_service_get_state(session->service);
 		state = service_to_session_state(service_state);
 		session->info->state = state;
 	}
@@ -1807,7 +1810,7 @@ static void session_activate(struct connman_session *session)
 
 		while (g_hash_table_iter_next(&iter, &key, &value)) {
 			struct connman_service_info *info = value;
-			state = __connman_service_get_state(info->service);
+			state = connman_service_get_state(info->service);
 
 			if (is_session_connected(session, state))
 				service_list = g_slist_prepend(service_list,
@@ -1836,7 +1839,7 @@ static void session_activate(struct connman_session *session)
 		struct connman_service_info *info = value;
 		enum connman_service_state state;
 
-		state = __connman_service_get_state(info->service);
+		state = connman_service_get_state(info->service);
 
 		if (is_session_connected(session, state) &&
 				session_match_service(session, info->service)) {
@@ -1998,7 +2001,7 @@ static void ipconfig_changed(struct connman_service *service,
 	}
 }
 
-static struct connman_notifier session_notifier = {
+static const struct connman_notifier session_notifier = {
 	.name			= "session",
 	.service_state_changed	= service_state_changed,
 	.ipconfig_changed	= ipconfig_changed,
