@@ -2,7 +2,7 @@
  *
  *  Connection Manager
  *
- *  Copyright (C) 2012  Intel Corporation. All rights reserved.
+ *  Copyright (C) 2012-2013  Intel Corporation. All rights reserved.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License version 2 as
@@ -406,7 +406,7 @@ static bool validate_ident(const char *ident)
 	return true;
 }
 
-static char *get_dir()
+static char *get_dir(void)
 {
 	return g_strdup_printf("%s", VPN_STORAGEDIR);
 }
@@ -484,12 +484,12 @@ static void config_notify_handler(struct inotify_event *event,
 	if (event->mask & IN_CREATE)
 		return;
 
-	if (event->mask & IN_DELETE) {
+	if (event->mask & (IN_DELETE | IN_MOVED_FROM)) {
 		g_hash_table_remove(config_table, ident);
 		return;
 	}
 
-	if (event->mask & IN_MODIFY) {
+	if (event->mask & (IN_MODIFY | IN_MOVED_TO)) {
 		struct vpn_config *config;
 		char *path = get_dir();
 
